(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.ClassTransformer = {}));
}(this, (function (exports) { 'use strict';

    /**
     * Used to create unique typed service identifier.
     * Useful when service has only interface, but don't have a class.
     */
    var Token = /** @class */ (function () {
        /**
         * @param name Token name, optional and only used for debugging purposes.
         */
        function Token(name) {
            this.name = name;
        }
        return Token;
    }());

    var __extends = (undefined && undefined.__extends) || (function () {
        var extendStatics = function (d, b) {
            extendStatics = Object.setPrototypeOf ||
                ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
                function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
            return extendStatics(d, b);
        };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    /**
     * Thrown when service is registered without type.
     */
    var MissingProvidedServiceTypeError = /** @class */ (function (_super) {
        __extends(MissingProvidedServiceTypeError, _super);
        function MissingProvidedServiceTypeError(identifier) {
            var _a, _b;
            var _this = _super.call(this) || this;
            _this.name = 'MissingProvidedServiceTypeError';
            /** Normalized identifier name used in the error message. */
            _this.normalizedIdentifier = '<UNKNOWN_IDENTIFIER>';
            if (typeof identifier === 'string') {
                _this.normalizedIdentifier = identifier;
            }
            else if (identifier instanceof Token) {
                _this.normalizedIdentifier = "Token<" + (identifier.name || 'UNSET_NAME') + ">";
            }
            else if (identifier && (identifier.name || ((_a = identifier.prototype) === null || _a === void 0 ? void 0 : _a.name))) {
                _this.normalizedIdentifier =
                    "MaybeConstructable<" + identifier.name + ">" ||
                        "MaybeConstructable<" + ((_b = identifier.prototype) === null || _b === void 0 ? void 0 : _b.name) + ">";
            }
            return _this;
        }
        Object.defineProperty(MissingProvidedServiceTypeError.prototype, "message", {
            get: function () {
                return "Cannot determine a class of the requesting service: \"" + this.normalizedIdentifier + "\".";
            },
            enumerable: false,
            configurable: true
        });
        return MissingProvidedServiceTypeError;
    }(Error));

    var __extends$1 = (undefined && undefined.__extends) || (function () {
        var extendStatics = function (d, b) {
            extendStatics = Object.setPrototypeOf ||
                ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
                function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
            return extendStatics(d, b);
        };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    /**
     * Thrown when requested service was not found.
     */
    var ServiceNotFoundError = /** @class */ (function (_super) {
        __extends$1(ServiceNotFoundError, _super);
        function ServiceNotFoundError(identifier) {
            var _a, _b;
            var _this = _super.call(this) || this;
            _this.name = 'ServiceNotFoundError';
            /** Normalized identifier name used in the error message. */
            _this.normalizedIdentifier = '<UNKNOWN_IDENTIFIER>';
            if (typeof identifier === 'string') {
                _this.normalizedIdentifier = identifier;
            }
            else if (identifier instanceof Token) {
                _this.normalizedIdentifier = "Token<" + (identifier.name || 'UNSET_NAME') + ">";
            }
            else if (identifier && (identifier.name || ((_a = identifier.prototype) === null || _a === void 0 ? void 0 : _a.name))) {
                _this.normalizedIdentifier =
                    "MaybeConstructable<" + identifier.name + ">" ||
                        "MaybeConstructable<" + ((_b = identifier.prototype) === null || _b === void 0 ? void 0 : _b.name) + ">";
            }
            return _this;
        }
        Object.defineProperty(ServiceNotFoundError.prototype, "message", {
            get: function () {
                return ("Service with \"" + this.normalizedIdentifier + "\" identifier was not found in the container. " +
                    "Register it before usage via explicitly calling the \"Container.set\" function or using the \"@Service()\" decorator.");
            },
            enumerable: false,
            configurable: true
        });
        return ServiceNotFoundError;
    }(Error));

    var EMPTY_VALUE = Symbol('EMPTY_VALUE');

    var __assign = (undefined && undefined.__assign) || function () {
        __assign = Object.assign || function(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                    t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };
    var __spreadArrays = (undefined && undefined.__spreadArrays) || function () {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    };
    /**
     * TypeDI can have multiple containers.
     * One container is ContainerInstance.
     */
    var ContainerInstance = /** @class */ (function () {
        function ContainerInstance(id) {
            /** All registered services in the container. */
            this.services = [];
            this.id = id;
        }
        ContainerInstance.prototype.has = function (identifier) {
            return !!this.findService(identifier);
        };
        ContainerInstance.prototype.get = function (identifier) {
            var globalContainer = Container.of(undefined);
            var globalService = globalContainer.findService(identifier);
            var scopedService = this.findService(identifier);
            if (globalService && globalService.global === true)
                return this.getServiceValue(globalService);
            if (scopedService)
                return this.getServiceValue(scopedService);
            /** If it's the first time requested in the child container we load it from parent and set it. */
            if (globalService && this !== globalContainer) {
                var clonedService = __assign({}, globalService);
                clonedService.value = EMPTY_VALUE;
                /**
                 * We need to immediately set the empty value from the root container
                 * to prevent infinite lookup in cyclic dependencies.
                 */
                this.set(clonedService);
                var value = this.getServiceValue(clonedService);
                this.set(__assign(__assign({}, clonedService), { value: value }));
                return value;
            }
            if (globalService)
                return this.getServiceValue(globalService);
            throw new ServiceNotFoundError(identifier);
        };
        ContainerInstance.prototype.getMany = function (id) {
            var _this = this;
            return this.filterServices(id).map(function (service) { return _this.getServiceValue(service); });
        };
        ContainerInstance.prototype.set = function (identifierOrServiceMetadata, value) {
            var _this = this;
            if (identifierOrServiceMetadata instanceof Array) {
                identifierOrServiceMetadata.forEach(function (data) { return _this.set(data); });
                return this;
            }
            if (typeof identifierOrServiceMetadata === 'string' || identifierOrServiceMetadata instanceof Token) {
                return this.set({
                    id: identifierOrServiceMetadata,
                    type: null,
                    value: value,
                    factory: undefined,
                    global: false,
                    multiple: false,
                    transient: false,
                });
            }
            if (typeof identifierOrServiceMetadata === 'function') {
                return this.set({
                    id: identifierOrServiceMetadata,
                    // TODO: remove explicit casting
                    type: identifierOrServiceMetadata,
                    value: value,
                    factory: undefined,
                    global: false,
                    multiple: false,
                    transient: false,
                });
            }
            var newService = __assign({ id: new Token('UNREACHABLE'), type: null, factory: undefined, value: EMPTY_VALUE, global: false, multiple: false, transient: false }, identifierOrServiceMetadata);
            var service = this.findService(newService.id);
            if (service && service.multiple !== true) {
                Object.assign(service, newService);
            }
            else {
                this.services.push(newService);
            }
            return this;
        };
        /**
         * Removes services with a given service identifiers (tokens or types).
         */
        ContainerInstance.prototype.remove = function () {
            var _this = this;
            var ids = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                ids[_i] = arguments[_i];
            }
            ids.forEach(function (id) {
                _this.filterServices(id).forEach(function (service) {
                    _this.services.splice(_this.services.indexOf(service), 1);
                });
            });
            return this;
        };
        /**
         * Completely resets the container by removing all previously registered services from it.
         */
        ContainerInstance.prototype.reset = function (options) {
            if (options === void 0) { options = { strategy: 'resetValue' }; }
            // TODO: The resources should be freed up.
            switch (options.strategy) {
                case 'resetValue':
                    this.services = this.services.map(function (s) {
                        return __assign(__assign({}, s), { value: !!s.type ? EMPTY_VALUE : s.value });
                    });
                    break;
                case 'resetValue':
                    this.services = [];
                    break;
                default:
                    throw new Error('Received invalid reset strategy.');
            }
            return this;
        };
        /**
         * Filters registered service in the with a given service identifier.
         */
        ContainerInstance.prototype.filterServices = function (identifier) {
            return this.services.filter(function (service) {
                if (service.id)
                    return service.id === identifier;
                if (service.type && typeof identifier === 'function')
                    return service.type === identifier || identifier.prototype instanceof service.type;
                return false;
            });
        };
        /**
         * Finds registered service in the with a given service identifier.
         */
        ContainerInstance.prototype.findService = function (identifier) {
            return this.services.find(function (service) {
                if (service.id) {
                    if (identifier instanceof Object &&
                        service.id instanceof Token &&
                        identifier.service instanceof Token) {
                        return service.id === identifier.service;
                    }
                    return service.id === identifier;
                }
                // todo: not sure why it was here || identifier.prototype instanceof service.type;
                if (service.type && typeof identifier === 'function') {
                    return service.type === identifier;
                }
                return false;
            });
        };
        /**
         * Gets service value.
         */
        ContainerInstance.prototype.getServiceValue = function (serviceMetadata) {
            // find if instance of this object already initialized in the container and return it if it is
            /**
             * If the service value has been set to anything prior to this call we return the set value.
             * NOTE: This part builds on the assumption that transient dependencies has no value set ever.
             */
            if (serviceMetadata.value !== EMPTY_VALUE) {
                return serviceMetadata.value;
            }
            // if named service was requested and its instance was not found plus there is not type to know what to initialize,
            // this means service was not pre-registered and we throw an exception
            if ((!serviceMetadata || !serviceMetadata.type) &&
                (!serviceMetadata || !serviceMetadata.factory) &&
                (typeof serviceMetadata.id === 'string' || serviceMetadata.id instanceof Token))
                throw new ServiceNotFoundError(serviceMetadata.id);
            // at this point we either have type in service registered, either identifier is a target type
            var construtableType;
            if (serviceMetadata && serviceMetadata.type) {
                construtableType = serviceMetadata.type;
            }
            else if (typeof serviceMetadata.id === 'function') {
                construtableType = serviceMetadata.id;
            }
            // setup constructor parameters for a newly initialized service
            var paramTypes = construtableType && Reflect && Reflect.getMetadata
                ? Reflect.getMetadata('design:paramtypes', construtableType)
                : undefined;
            // TODO: remove as any, here it cannot be undefined but TS doesn't see this.
            var params = paramTypes ? this.initializeParams(construtableType, paramTypes) : [];
            // if factory is set then use it to create service instance
            var value;
            if (serviceMetadata.factory) {
                // filter out non-service parameters from created service constructor
                // non-service parameters can be, lets say Car(name: string, isNew: boolean, engine: Engine)
                // where name and isNew are non-service parameters and engine is a service parameter
                params = params.filter(function (param) { return param !== undefined; });
                if (serviceMetadata.factory instanceof Array) {
                    // use special [Type, "create"] syntax to allow factory services
                    // in this case Type instance will be obtained from Container and its method "create" will be called
                    var factoryInstance = void 0;
                    try {
                        factoryInstance = this.get(serviceMetadata.factory[0]);
                    }
                    catch (error) {
                        if (error instanceof ServiceNotFoundError) {
                            factoryInstance = new serviceMetadata.factory[0]();
                        }
                    }
                    // TODO: rethink this
                    // We need to pass `this` to make the following test pass:
                    // "should support function injection with Token dependencies"
                    // We should have the dependencies here, instead of passing the container to the factory.
                    // Maybe we should save the dependency ID list on the meta-data?
                    value = factoryInstance[serviceMetadata.factory[1]].apply(factoryInstance, __spreadArrays(params, [this]));
                }
                else {
                    // regular factory function
                    value = serviceMetadata.factory.apply(serviceMetadata, __spreadArrays(params, [this]));
                }
            }
            else {
                // TODO: Commented it out as this makes no sense.
                // params.unshift(null);
                // "extra feature" - always pass container instance as the last argument to the service function
                // this allows us to support javascript where we don't have decorators and emitted metadata about dependencies
                // need to be injected, and user can use provided container to get instances he needs
                params.push(this);
                if (!construtableType)
                    throw new MissingProvidedServiceTypeError(serviceMetadata.id);
                // eslint-disable-next-line prefer-spread
                value = new (construtableType.bind.apply(construtableType, __spreadArrays([void 0], params)))();
            }
            if (serviceMetadata && !serviceMetadata.transient && value)
                serviceMetadata.value = value;
            if (construtableType)
                this.applyPropertyHandlers(construtableType, value);
            return value;
        };
        /**
         * Initializes all parameter types for a given target service class.
         */
        ContainerInstance.prototype.initializeParams = function (target, paramTypes) {
            var _this = this;
            return paramTypes.map(function (paramType, index) {
                var paramHandler = Container.handlers.find(function (handler) {
                    /**
                     * @Inject()-ed values are stored as parameter handlers and they reference their target
                     * when created. So when a class is extended the @Inject()-ed values are not inherited
                     * because the handler still points to the old object only.
                     *
                     * As a quick fix a single level parent lookup is added via `Object.getPrototypeOf(target)`,
                     * however this should be updated to a more robust solution.
                     *
                     * TODO: Add proper inheritance handling: either copy the handlers when a class is registered what
                     * TODO: has it's parent already registered as dependency or make the lookup search up to the base Object.
                     */
                    return ((handler.object === target || handler.object === Object.getPrototypeOf(target)) && handler.index === index);
                });
                if (paramHandler)
                    return paramHandler.value(_this);
                if (paramType && paramType.name && !_this.isTypePrimitive(paramType.name)) {
                    return _this.get(paramType);
                }
                return undefined;
            });
        };
        /**
         * Checks if given type is primitive (e.g. string, boolean, number, object).
         */
        ContainerInstance.prototype.isTypePrimitive = function (param) {
            return ['string', 'boolean', 'number', 'object'].indexOf(param.toLowerCase()) !== -1;
        };
        /**
         * Applies all registered handlers on a given target class.
         */
        ContainerInstance.prototype.applyPropertyHandlers = function (target, instance) {
            var _this = this;
            Container.handlers.forEach(function (handler) {
                if (typeof handler.index === 'number')
                    return;
                if (handler.object.constructor !== target && !(target.prototype instanceof handler.object.constructor))
                    return;
                if (handler.propertyName) {
                    instance[handler.propertyName] = handler.value(_this);
                }
            });
        };
        return ContainerInstance;
    }());

    /**
     * Service container.
     */
    var Container = /** @class */ (function () {
        function Container() {
        }
        /**
         * Gets a separate container instance for the given instance id.
         */
        Container.of = function (containerId) {
            if (containerId === void 0) { containerId = 'default'; }
            if (containerId === 'default')
                return this.globalInstance;
            var container = this.instances.find(function (instance) { return instance.id === containerId; });
            if (!container) {
                container = new ContainerInstance(containerId);
                this.instances.push(container);
                // TODO: Why we are not reseting here? Let's reset here. (I have added the commented code.)
                // container.reset();
            }
            return container;
        };
        Container.has = function (identifier) {
            return this.globalInstance.has(identifier);
        };
        Container.get = function (identifier) {
            return this.globalInstance.get(identifier);
        };
        Container.getMany = function (id) {
            return this.globalInstance.getMany(id);
        };
        Container.set = function (identifierOrServiceMetadata, value) {
            this.globalInstance.set(identifierOrServiceMetadata, value);
            return this;
        };
        /**
         * Removes services with a given service identifiers (tokens or types).
         */
        Container.remove = function () {
            var _a;
            var ids = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                ids[_i] = arguments[_i];
            }
            (_a = this.globalInstance).remove.apply(_a, ids);
            return this;
        };
        /**
         * Completely resets the container by removing all previously registered services and handlers from it.
         */
        Container.reset = function (containerId) {
            if (containerId === void 0) { containerId = 'default'; }
            if (containerId == 'default') {
                this.globalInstance.reset();
                this.instances.forEach(function (instance) { return instance.reset(); });
            }
            else {
                var instance = this.instances.find(function (instance) { return instance.id === containerId; });
                if (instance) {
                    instance.reset();
                    this.instances.splice(this.instances.indexOf(instance), 1);
                }
            }
            return this;
        };
        /**
         * Registers a new handler.
         */
        Container.registerHandler = function (handler) {
            this.handlers.push(handler);
            return this;
        };
        /**
         * Helper method that imports given services.
         */
        /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
        Container.import = function (services) {
            return this;
        };
        /**
         * All registered handlers. The @Inject() decorator uses handlers internally to mark a property for injection.
         **/
        Container.handlers = [];
        /**  Global container instance. */
        Container.globalInstance = new ContainerInstance('default');
        /** Other containers created using Container.of method. */
        Container.instances = [];
        return Container;
    }());

    /**
     * Marks class as a service that can be injected using container.
     */
    function Service(optionsOrServiceName, maybeFactory) {
        if (arguments.length === 2 || typeof optionsOrServiceName === 'function') {
            var serviceId = new Token();
            var dependencies_1 = arguments.length === 2 ? optionsOrServiceName : [];
            var factory_1 = arguments.length === 2 ? maybeFactory : optionsOrServiceName;
            if (factory_1 === undefined) {
                // TODO: This should never happen, but regardless, lets re-think this bit.
                throw Error('Ohh no.');
            }
            Container.set({
                id: serviceId,
                factory: [
                    /** @class */ (function () {
                        function DefaultFactory() {
                        }
                        DefaultFactory.prototype.create = function (container) {
                            var params = dependencies_1.map(function (dependency) { return container.get(dependency); });
                            return factory_1.apply(void 0, params);
                        };
                        return DefaultFactory;
                    }()),
                    'create',
                ],
                global: false,
                multiple: false,
                transient: false,
                value: EMPTY_VALUE,
            });
            return serviceId;
        }
        else {
            return function (target) {
                var service = {
                    id: target,
                    type: target,
                    factory: undefined,
                    multiple: false,
                    global: false,
                    transient: false,
                    value: EMPTY_VALUE,
                };
                if (typeof optionsOrServiceName === 'string' || optionsOrServiceName instanceof Token) {
                    service.id = optionsOrServiceName;
                }
                else if (optionsOrServiceName) {
                    // ServiceOptions
                    service.id = optionsOrServiceName.id || target;
                    service.factory = optionsOrServiceName.factory || undefined;
                    service.multiple = optionsOrServiceName.multiple || false;
                    service.global = optionsOrServiceName.global || false;
                    service.transient = optionsOrServiceName.transient || false;
                }
                Container.set(service);
            };
        }
    }

    var __extends$2 = (undefined && undefined.__extends) || (function () {
        var extendStatics = function (d, b) {
            extendStatics = Object.setPrototypeOf ||
                ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
                function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
            return extendStatics(d, b);
        };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    /**
     * Thrown when DI cannot inject value into property decorated by @Inject decorator.
     */
    var CannotInjectValueError = /** @class */ (function (_super) {
        __extends$2(CannotInjectValueError, _super);
        function CannotInjectValueError(target, propertyName) {
            var _this = _super.call(this) || this;
            _this.target = target;
            _this.propertyName = propertyName;
            _this.name = 'CannotInjectValueError';
            return _this;
        }
        Object.defineProperty(CannotInjectValueError.prototype, "message", {
            get: function () {
                return ("Cannot inject value into \"" + this.target.constructor.name + "." + this.propertyName + "\". " +
                    "Please make sure you setup reflect-metadata properly and you don't use interfaces without service tokens as injection value.");
            },
            enumerable: false,
            configurable: true
        });
        return CannotInjectValueError;
    }(Error));

    /**
     * Injects a service into a class property or constructor parameter.
     */
    function Inject(typeOrName) {
        return function (target, propertyName, index) {
            if (!typeOrName)
                typeOrName = function () { return Reflect.getMetadata('design:type', target, propertyName); };
            Container.registerHandler({
                object: target,
                propertyName: propertyName,
                index: index,
                value: function (containerInstance) {
                    var identifier;
                    if (typeof typeOrName === 'string') {
                        identifier = typeOrName;
                    }
                    else if (typeOrName instanceof Token) {
                        identifier = typeOrName;
                    }
                    else {
                        identifier = typeOrName();
                    }
                    if (identifier === Object)
                        throw new CannotInjectValueError(target, propertyName);
                    return containerInstance.get(identifier);
                },
            });
        };
    }

    /**
     * Injects a service into a class property or constructor parameter.
     */
    function InjectMany(typeOrName) {
        return function (target, propertyName, index) {
            if (!typeOrName)
                typeOrName = function () { return Reflect.getMetadata('design:type', target, propertyName); };
            Container.registerHandler({
                object: target,
                propertyName: propertyName,
                index: index,
                value: function (containerInstance) {
                    var identifier;
                    if (typeof typeOrName === 'string') {
                        identifier = typeOrName;
                    }
                    else if (typeOrName instanceof Token) {
                        identifier = typeOrName;
                    }
                    else {
                        identifier = typeOrName();
                    }
                    if (identifier === Object)
                        throw new CannotInjectValueError(target, propertyName);
                    return containerInstance.getMany(identifier);
                },
            });
        };
    }

    exports.Container = Container;
    exports.ContainerInstance = ContainerInstance;
    exports.Inject = Inject;
    exports.InjectMany = InjectMany;
    exports.Service = Service;
    exports.Token = Token;
    exports.default = Container;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=typedi.umd.js.map
