"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Inject = void 0;
const container_class_1 = require("../container.class");
const token_class_1 = require("../token.class");
const cannot_inject_value_error_1 = require("../error/cannot-inject-value.error");
/**
 * Injects a service into a class property or constructor parameter.
 */
function Inject(typeOrName) {
    return function (target, propertyName, index) {
        if (!typeOrName)
            typeOrName = () => Reflect.getMetadata('design:type', target, propertyName);
        container_class_1.Container.registerHandler({
            object: target,
            propertyName: propertyName,
            index: index,
            value: containerInstance => {
                let identifier;
                if (typeof typeOrName === 'string') {
                    identifier = typeOrName;
                }
                else if (typeOrName instanceof token_class_1.Token) {
                    identifier = typeOrName;
                }
                else {
                    identifier = typeOrName();
                }
                if (identifier === Object)
                    throw new cannot_inject_value_error_1.CannotInjectValueError(target, propertyName);
                return containerInstance.get(identifier);
            },
        });
    };
}
exports.Inject = Inject;
//# sourceMappingURL=inject.decorator.js.map