"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Service = void 0;
const container_class_1 = require("../container.class");
const token_class_1 = require("../token.class");
const empty_const_1 = require("../empty.const");
/**
 * Marks class as a service that can be injected using container.
 */
function Service(optionsOrServiceName, maybeFactory) {
    if (arguments.length === 2 || typeof optionsOrServiceName === 'function') {
        const serviceId = new token_class_1.Token();
        const dependencies = arguments.length === 2 ? optionsOrServiceName : [];
        const factory = arguments.length === 2 ? maybeFactory : optionsOrServiceName;
        if (factory === undefined) {
            // TODO: This should never happen, but regardless, lets re-think this bit.
            throw Error('Ohh no.');
        }
        container_class_1.Container.set({
            id: serviceId,
            factory: [
                class DefaultFactory {
                    create(container) {
                        const params = dependencies.map(dependency => container.get(dependency));
                        return factory(...params);
                    }
                },
                'create',
            ],
            global: false,
            multiple: false,
            transient: false,
            value: empty_const_1.EMPTY_VALUE,
        });
        return serviceId;
    }
    else {
        return function (target) {
            const service = {
                id: target,
                type: target,
                factory: undefined,
                multiple: false,
                global: false,
                transient: false,
                value: empty_const_1.EMPTY_VALUE,
            };
            if (typeof optionsOrServiceName === 'string' || optionsOrServiceName instanceof token_class_1.Token) {
                service.id = optionsOrServiceName;
            }
            else if (optionsOrServiceName) {
                // ServiceOptions
                service.id = optionsOrServiceName.id || target;
                service.factory = optionsOrServiceName.factory || undefined;
                service.multiple = optionsOrServiceName.multiple || false;
                service.global = optionsOrServiceName.global || false;
                service.transient = optionsOrServiceName.transient || false;
            }
            container_class_1.Container.set(service);
        };
    }
}
exports.Service = Service;
//# sourceMappingURL=service.decorator.js.map