import { Container } from '../container.class';
import { Token } from '../token.class';
import { CannotInjectValueError } from '../error/cannot-inject-value.error';
/**
 * Injects a service into a class property or constructor parameter.
 */
export function InjectMany(typeOrName) {
    return function (target, propertyName, index) {
        if (!typeOrName)
            typeOrName = () => Reflect.getMetadata('design:type', target, propertyName);
        Container.registerHandler({
            object: target,
            propertyName: propertyName,
            index: index,
            value: containerInstance => {
                let identifier;
                if (typeof typeOrName === 'string') {
                    identifier = typeOrName;
                }
                else if (typeOrName instanceof Token) {
                    identifier = typeOrName;
                }
                else {
                    identifier = typeOrName();
                }
                if (identifier === Object)
                    throw new CannotInjectValueError(target, propertyName);
                return containerInstance.getMany(identifier);
            },
        });
    };
}
//# sourceMappingURL=inject-many.decorator.js.map