import { Container } from '../container.class';
import { Token } from '../token.class';
import { EMPTY_VALUE } from '../empty.const';
/**
 * Marks class as a service that can be injected using container.
 */
export function Service(optionsOrServiceName, maybeFactory) {
    if (arguments.length === 2 || typeof optionsOrServiceName === 'function') {
        const serviceId = new Token();
        const dependencies = arguments.length === 2 ? optionsOrServiceName : [];
        const factory = arguments.length === 2 ? maybeFactory : optionsOrServiceName;
        if (factory === undefined) {
            // TODO: This should never happen, but regardless, lets re-think this bit.
            throw Error('Ohh no.');
        }
        Container.set({
            id: serviceId,
            factory: [
                class DefaultFactory {
                    create(container) {
                        const params = dependencies.map(dependency => container.get(dependency));
                        return factory(...params);
                    }
                },
                'create',
            ],
            global: false,
            multiple: false,
            transient: false,
            value: EMPTY_VALUE,
        });
        return serviceId;
    }
    else {
        return function (target) {
            const service = {
                id: target,
                type: target,
                factory: undefined,
                multiple: false,
                global: false,
                transient: false,
                value: EMPTY_VALUE,
            };
            if (typeof optionsOrServiceName === 'string' || optionsOrServiceName instanceof Token) {
                service.id = optionsOrServiceName;
            }
            else if (optionsOrServiceName) {
                // ServiceOptions
                service.id = optionsOrServiceName.id || target;
                service.factory = optionsOrServiceName.factory || undefined;
                service.multiple = optionsOrServiceName.multiple || false;
                service.global = optionsOrServiceName.global || false;
                service.transient = optionsOrServiceName.transient || false;
            }
            Container.set(service);
        };
    }
}
//# sourceMappingURL=service.decorator.js.map