import { Container } from '../container.class';
import { Token } from '../token.class';
import { CannotInjectValueError } from '../error/cannot-inject-value.error';
/**
 * Injects a service into a class property or constructor parameter.
 */
export function Inject(typeOrName) {
    return function (target, propertyName, index) {
        if (!typeOrName)
            typeOrName = function () { return Reflect.getMetadata('design:type', target, propertyName); };
        Container.registerHandler({
            object: target,
            propertyName: propertyName,
            index: index,
            value: function (containerInstance) {
                var identifier;
                if (typeof typeOrName === 'string') {
                    identifier = typeOrName;
                }
                else if (typeOrName instanceof Token) {
                    identifier = typeOrName;
                }
                else {
                    identifier = typeOrName();
                }
                if (identifier === Object)
                    throw new CannotInjectValueError(target, propertyName);
                return containerInstance.get(identifier);
            },
        });
    };
}
//# sourceMappingURL=inject.decorator.js.map