import { Container } from '../container.class';
import { Token } from '../token.class';
import { EMPTY_VALUE } from '../empty.const';
/**
 * Marks class as a service that can be injected using container.
 */
export function Service(optionsOrServiceName, maybeFactory) {
    if (arguments.length === 2 || typeof optionsOrServiceName === 'function') {
        var serviceId = new Token();
        var dependencies_1 = arguments.length === 2 ? optionsOrServiceName : [];
        var factory_1 = arguments.length === 2 ? maybeFactory : optionsOrServiceName;
        if (factory_1 === undefined) {
            // TODO: This should never happen, but regardless, lets re-think this bit.
            throw Error('Ohh no.');
        }
        Container.set({
            id: serviceId,
            factory: [
                /** @class */ (function () {
                    function DefaultFactory() {
                    }
                    DefaultFactory.prototype.create = function (container) {
                        var params = dependencies_1.map(function (dependency) { return container.get(dependency); });
                        return factory_1.apply(void 0, params);
                    };
                    return DefaultFactory;
                }()),
                'create',
            ],
            global: false,
            multiple: false,
            transient: false,
            value: EMPTY_VALUE,
        });
        return serviceId;
    }
    else {
        return function (target) {
            var service = {
                id: target,
                type: target,
                factory: undefined,
                multiple: false,
                global: false,
                transient: false,
                value: EMPTY_VALUE,
            };
            if (typeof optionsOrServiceName === 'string' || optionsOrServiceName instanceof Token) {
                service.id = optionsOrServiceName;
            }
            else if (optionsOrServiceName) {
                // ServiceOptions
                service.id = optionsOrServiceName.id || target;
                service.factory = optionsOrServiceName.factory || undefined;
                service.multiple = optionsOrServiceName.multiple || false;
                service.global = optionsOrServiceName.global || false;
                service.transient = optionsOrServiceName.transient || false;
            }
            Container.set(service);
        };
    }
}
//# sourceMappingURL=service.decorator.js.map